Attribute VB_Name = "REGAPI"
Option Explicit

'Registry Constants
Public Const HKCR = &H80000000
Public Const HKCU = &H80000001
Public Const HKLM = &H80000002
Public Const HKU = &H80000003

'Registry Specific Access Rights
Private Const KEY_QUERY_VALUE = &H1
Private Const KEY_SET_VALUE = &H2
Private Const KEY_CREATE_SUB_KEY = &H4
Private Const KEY_ENUMERATE_SUB_KEYS = &H8
Private Const KEY_NOTIFY = &H10
Private Const KEY_CREATE_LINK = &H20
Private Const KEY_ALL_ACCESS = &H3F

'Open/Create Options
Private Const REG_OPTION_NON_VOLATILE = 0&
Private Const REG_OPTION_VOLATILE = &H1

'Key creation/open disposition
Private Const REG_CREATED_NEW_KEY = &H1
Private Const REG_OPENED_EXISTING_KEY = &H2

'Masks for the predefined standard access types
Private Const STANDARD_RIGHTS_ALL = &H1F0000
Private Const SPECIFIC_RIGHTS_ALL = &HFFFF

'Define severity codes
Private Const ERROR_SUCCESS = 0&
Private Const ERROR_ACCESS_DENIED = 5
Private Const ERROR_NO_MORE_ITEMS = 259

'Predefined Value Types
Private Const REG_NONE = (0)                         'No value type
Private Const REG_SZ = (1)                           'Unicode nul terminated string
Private Const REG_EXPAND_SZ = (2)                    'Unicode nul terminated string w/enviornment var
Private Const REG_BINARY = (3)                       'Free form binary
Private Const REG_DWORD = (4)                        '32-bit number
Private Const REG_DWORD_LITTLE_ENDIAN = (4)          '32-bit number (same as REG_DWORD)
Private Const REG_DWORD_BIG_ENDIAN = (5)             '32-bit number
Private Const REG_LINK = (6)                         'Symbolic Link (unicode)
Private Const REG_MULTI_SZ = (7)                     'Multiple Unicode strings
Private Const REG_RESOURCE_LIST = (8)                'Resource list in the resource map
Private Const REG_FULL_RESOURCE_DESCRIPTOR = (9)     'Resource list in the hardware description
Private Const REG_RESOURCE_REQUIREMENTS_LIST = (10)


'Structures Needed For Registry Prototypes
Private Type SECURITY_ATTRIBUTES
  nLength As Long
  lpSecurityDescriptor As Long
  bInheritHandle As Boolean
End Type

Private Type FILETIME
  dwLowDateTime As Long
  dwHighDateTime As Long
End Type

'Registry Function Prototypes
Private Declare Function RegOpenKeyEx Lib "advapi32" Alias "RegOpenKeyExA" _
  (ByVal hKey As Long, ByVal lpSubKey As String, ByVal ulOptions As Long, _
  ByVal samDesired As Long, phkResult As Long) As Long

Private Declare Function RegSetValueEx Lib "advapi32" Alias "RegSetValueExA" _
  (ByVal hKey As Long, ByVal lpValueName As String, ByVal Reserved As Long, _
   ByVal dwType As Long, ByVal szData As String, ByVal cbData As Long) As Long

Private Declare Function RegCloseKey Lib "advapi32" (ByVal hKey As Long) As Long

Private Declare Function RegQueryValueEx Lib "advapi32" Alias "RegQueryValueExA" _
  (ByVal hKey As Long, ByVal lpValueName As String, ByVal lpReserved As Long, _
   ByRef lpType As Long, ByVal szData As String, ByRef lpcbData As Long) As Long

Private Declare Function RegCreateKeyEx Lib "advapi32" Alias "RegCreateKeyExA" _
  (ByVal hKey As Long, ByVal lpSubKey As String, ByVal Reserved As Long, _
   ByVal lpClass As String, ByVal dwOptions As Long, ByVal samDesired As Long, _
   lpSecurityAttributes As SECURITY_ATTRIBUTES, phkResult As Long, _
   lpdwDisposition As Long) As Long

Private Declare Function RegEnumKeyEx Lib "advapi32.dll" Alias "RegEnumKeyExA" _
  (ByVal hKey As Long, ByVal dwIndex As Long, ByVal lpName As String, _
   lpcbName As Long, ByVal lpReserved As Long, ByVal lpClass As String, _
   lpcbClass As Long, lpftLastWriteTime As FILETIME) As Long

Private Declare Function RegEnumValue Lib "advapi32.dll" Alias "RegEnumValueA" _
  (ByVal hKey As Long, ByVal dwIndex As Long, ByVal lpValueName As String, _
   lpcbValueName As Long, ByVal lpReserved As Long, lpType As Long, _
   ByVal lpData As String, lpcbData As Long) As Long

Private Declare Function RegDeleteKeyA Lib "advapi32.dll" _
  (ByVal hKey As Long, ByVal lpSubKey As String) As Long

Private Declare Function RegDeleteValueA Lib "advapi32.dll" _
  (ByVal hKey As Long, ByVal lpValueName As String) As Long
Public Function REGSaveSetting(ByVal hKey&, ByVal Subkey$, ByVal Value$, ByVal Setting$) As Boolean
'--------------------------------------------------------------------------
'RegSaveSetting uses the 32bit registry APIs to Save Registry Settings and
'   returns TRUE if successful, FALSE if unsuccessful.
'
'Parameters:    hKey& - Root Key Name
'               SubKey$ - Delimited (\) Sub Key Name
'               Value$ - Registry Entry Value Name
'               Setting$ - Value Setting
'
'Note:  This Function writes string values only.
'
'--------------------------------------------------------------------------

Dim Create As Long, phkResult As Long, lResult As Long, SA As SECURITY_ATTRIBUTES

    If Setting$ <> "" Then
        RegCreateKeyEx hKey&, Subkey$, 0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, SA, phkResult, Create
        lResult = RegSetValueEx(phkResult, Value$, 0, REG_SZ, Setting$, CLng(Len(Setting$) + 1))
        RegCloseKey phkResult
      Else 'Empty Value
        RegCreateKeyEx hKey&, Subkey$, 0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, SA, phkResult, Create
        lResult = RegSetValueEx(phkResult, Value$, 0, REG_SZ, "", 1&)
        RegCloseKey phkResult
    End If
        
    REGSaveSetting = (lResult = ERROR_SUCCESS)

End Function
Public Function REGGetSetting(ByVal hKey&, ByVal Subkey$, ByVal Value$, Optional Default) As String
'--------------------------------------------------------------------------
'RegGetSetting uses the 32bit registry APIs to retrieve registry settings.
'   If the setting does not exist, Default is returned (if specified),
'   otherwise NULL is returned.
'
'Parameters:    hKey& - Root Key Name
'               SubKey$ - Delimited (\) Sub Key Name
'               Value$ - Registry Entry Value
'               Default - Optional Default Setting
'
'Note:  This Function returns string values only, regardless of the data
'       type queried; non-string values are returned as random garbage.
'
'--------------------------------------------------------------------------

Dim phkResult As Long, lResult As Long, szBuffer As String, lBuffSize As Long, szDefault As String

szBuffer = Space(255)
lBuffSize = Len(szBuffer)
    
    RegOpenKeyEx hKey, Subkey$, 0, 1, phkResult
    lResult = RegQueryValueEx(phkResult, Value$, 0, 0, szBuffer, lBuffSize)
    RegCloseKey phkResult
    
    If lResult = ERROR_SUCCESS Then
        REGGetSetting = Left(szBuffer, lBuffSize - 1)
      Else
        If szDefault <> "" Then
            REGGetSetting = szDefault
          Else
            If Not IsMissing(Default) Then REGGetSetting = CStr(Default)
        End If
    End If

End Function
Public Static Function REGConfirmKey(ByVal hKey&, ByVal Subkey$) As Boolean
'--------------------------------------------------------------------------
'RegConfirmKey uses the 32bit registry APIs to confirm the existence of the
'   specified registry key.  If the key exists, the function returns TRUE
'   or FALSE if the key does not exist.
'
'Parameters:    hKey& - Root Key Name
'               SubKey$ - Delimited (\) Sub Key Name
'
'--------------------------------------------------------------------------
On Error GoTo ErrorHandle

Dim phkResult As Long

    RegOpenKeyEx hKey, Subkey$, 0, 1, phkResult
        If phkResult <> 0 Then
            REGConfirmKey = True
        End If
    RegCloseKey phkResult

Exit Function

ErrorHandle:
    REGConfirmKey = False

End Function
Public Function REGDeleteValue(ByVal hKey&, ByVal Subkey$, ByVal Value$) As Boolean
'--------------------------------------------------------------------------
'RegDeleteValue uses the 32bit registry APIs to Delete Registry Values and
'   returns TRUE if successful, FALSE if unsuccessful, or the specified
'   value doesn't exist.
'
'Parameters:    hKey& - Root Key Name
'               SubKey$ - Delimited (\) Sub Key Name
'               Value$ - Registry Entry Value Name
'
'--------------------------------------------------------------------------

Dim Create As Long, phkResult As Long, SA As SECURITY_ATTRIBUTES

    RegCreateKeyEx hKey&, Subkey$, 0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, SA, phkResult, Create
    If RegDeleteValueA(phkResult, Value$) = ERROR_SUCCESS Then REGDeleteValue = True
    RegCloseKey phkResult

End Function
Public Function REGDeleteKey(ByVal hKey&, ByVal Subkey$, ByVal KeyToDelete$) As Boolean
'--------------------------------------------------------------------------
'RegDeleteKey uses the 32bit registry APIs to Delete a Registry Key and
'   returns TRUE if successful, FALSE if unsuccessful, or the specified
'   Key doesn't exist.
'
'   Note: KeyToDelete$ must not have subkeys or the function will fail and
'         return FALSE.
'
'Parameters:    hKey& - Root Key Name
'               SubKey$ - Delimited (\) Sub Key Name
'               KeyToDelete$ - Registry Key to Delete
'
'--------------------------------------------------------------------------

Dim Create As Long, phkResult As Long, SA As SECURITY_ATTRIBUTES

    RegCreateKeyEx hKey&, Subkey$, 0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, SA, phkResult, Create

    If RegDeleteKeyA(phkResult, KeyToDelete$) = ERROR_SUCCESS Then
        REGDeleteKey = True
      Else
        RegCloseKey phkResult
    End If
    
End Function
